@@ -38,37 +38,38 @@ except ImportError:
 import token
 import tokenize
 from optparse import OptionParser
-from subprocess import Popen, PIPE
 import difflib
 import tempfile
 
-from distutils.version import StrictVersion
 try:
     import pep8
-    try:
-        if StrictVersion(pep8.__version__) < StrictVersion('1.3a2'):
-            pep8 = None
-    except ValueError:
-        # Ignore non-standard version tags.
-        pass
 except ImportError:
     pep8 = None
 
 
-__version__ = '0.8.6'
-
-
-PEP8_BIN = 'pep8'
-CR = '\r'
-LF = '\n'
-CRLF = '\r\n'
-
 try:
     unicode
 except NameError:
     unicode = str
 
 
+__version__ = '0.8.7'
+
+
+CR = '\r'
+LF = '\n'
+CRLF = '\r\n'
+
+
+# For generating line shortening candidates.
+SHORTEN_OPERATOR_GROUPS = frozenset([
+    frozenset([',']),
+    frozenset(['%']),
+    frozenset([',', '(', '[', '{']),
+    frozenset([',', '(', '[', '{', '%', '+', '-', '*', '/', '//']),
+])
+
+
 def open_with_encoding(filename, encoding=None, mode='r'):
     """Return opened file with a specific encoding."""
     if not encoding:
@@ -163,6 +164,8 @@ class FixPEP8(object):
         self.fix_e222 = self.fix_e271
         self.fix_e223 = self.fix_e271
         self.fix_e226 = self.fix_e225
+        self.fix_e227 = self.fix_e225
+        self.fix_e228 = self.fix_e225
         self.fix_e241 = self.fix_e271
         self.fix_e242 = self.fix_e224
         self.fix_e261 = self.fix_e262
@@ -219,38 +222,12 @@ class FixPEP8(object):
 
     def fix(self):
         """Return a version of the source code with PEP 8 violations fixed."""
-        if pep8:
-            pep8_options = {
-                'ignore': self.options.ignore,
-                'select': self.options.select,
-                'max_line_length': self.options.max_line_length,
-            }
-            results = _execute_pep8(pep8_options, self.source)
-        else:
-            encoding = detect_encoding(self.filename)
-
-            (_tmp_open_file, tmp_filename) = tempfile.mkstemp()
-            os.close(_tmp_open_file)
-            fp = open_with_encoding(tmp_filename, encoding=encoding, mode='w')
-            fp.write(unicode().join(self.source))
-            fp.close()
-
-            if self.options.verbose:
-                print('Running in compatibility mode. Consider '
-                      'upgrading to the latest pep8.',
-                      file=sys.stderr)
-            results = _spawn_pep8((['--ignore=' +
-                                    ','.join(self.options.ignore)]
-                                   if self.options.ignore else []) +
-                                  (['--select=' +
-                                    ','.join(self.options.select)]
-                                   if self.options.select else []) +
-                                  (['--max-line-length={length}'.format(
-                                      length=self.options.max_line_length)]
-                                   if self.options.max_line_length else []) +
-                                  [tmp_filename])
-            if not pep8:
-                os.remove(tmp_filename)
+        pep8_options = {
+            'ignore': self.options.ignore,
+            'select': self.options.select,
+            'max_line_length': self.options.max_line_length,
+        }
+        results = _execute_pep8(pep8_options, self.source)
 
         if self.options.verbose:
             progress = {}
@@ -685,7 +662,8 @@ class FixPEP8(object):
 
         indent = _get_indentation(target)
         source = target[len(indent):]
-        sio = StringIO(target)
+        assert source.lstrip() == source
+        sio = StringIO(source)
 
         # Check for multiline string.
         try:
@@ -693,8 +671,7 @@ class FixPEP8(object):
         except (tokenize.TokenError, IndentationError):
             multi_line_candidate = break_multi_line(
                 target, newline=self.newline,
-                indent_word=self.indent_word,
-                max_line_length=self.options.max_line_length)
+                indent_word=self.indent_word)
 
             if multi_line_candidate:
                 self.source[line_index] = multi_line_candidate
@@ -702,34 +679,36 @@ class FixPEP8(object):
             else:
                 return []
 
-        # Prefer
-        # my_long_function_name(
-        #     x, y, z, ...)
-        #
-        # over
-        # my_long_function_name(x, y,
-        #     z, ...)
-        candidates = [None, None]
-        for candidate_index, reverse in enumerate([False, True]):
-            candidates[candidate_index] = shorten_line(
-                tokens, source, target, indent,
-                self.indent_word, newline=self.newline,
-                max_line_length=self.options.max_line_length,
-                reverse=reverse,
-                aggressive=self.options.aggressive)
+        candidates = shorten_line(
+            tokens, source, indent,
+            self.indent_word, newline=self.newline,
+            aggressive=self.options.aggressive)
 
-        if candidates[0] and candidates[1]:
-            if candidates[0].split(self.newline)[0].endswith('('):
-                self.source[line_index] = candidates[0]
-            else:
-                self.source[line_index] = candidates[1]
-        elif candidates[0]:
-            self.source[line_index] = candidates[0]
-        elif candidates[1]:
-            self.source[line_index] = candidates[1]
-        else:
-            # Otherwise both don't work
-            return []
+        candidates = list(sorted(
+            set(candidates),
+            key=lambda x: line_shortening_rank(x,
+                                               self.newline,
+                                               self.indent_word)))
+
+        if self.options.verbose >= 4:
+            print(('-' * 79 + '\n').join([''] + candidates + ['']),
+                  file=sys.stderr)
+
+        for _candidate in candidates:
+            if _candidate is None:
+                continue
+
+            if _candidate == target:
+                continue
+
+            if (get_longest_length(_candidate, self.newline) >=
+                    get_longest_length(target, self.newline)):
+                continue
+
+            self.source[line_index] = _candidate
+            return
+
+        return []
 
     def fix_e502(self, result):
         """Remove extraneous escape of newline."""
@@ -944,16 +923,6 @@ def _get_indentation(line):
         return ''
 
 
-def _analyze_pep8result(result):
-    tmp = result.split(':')
-    info = ' '.join(result.split()[1:])
-    return {'id': info.lstrip().split()[0],
-            'filename': tmp[0],
-            'line': int(tmp[1]),
-            'column': int(tmp[2]),
-            'info': info}
-
-
 def _get_difftext(old, new, filename):
     diff = difflib.unified_diff(
         old, new,
@@ -984,68 +953,50 @@ def _priority_key(pep8_result):
         return len(priority)
 
 
-def shorten_line(tokens, source, target, indentation, indent_word, newline,
-                 max_line_length, reverse=False, aggressive=False):
-    """Separate line at OPERATOR."""
-    actual_length = len(indentation) + len(source)
+def shorten_line(tokens, source, indentation, indent_word, newline,
+                 aggressive=False):
+    """Separate line at OPERATOR.
 
-    delta = (actual_length - max_line_length) // 3
-    assert delta >= 0
+    Multiple candidates will be yielded.
 
-    if not delta:
-        delta = 1
+    """
+    for candidate in _shorten_line(tokens=tokens,
+                                   source=source,
+                                   indentation=indentation,
+                                   indent_word=indent_word,
+                                   newline=newline,
+                                   aggressive=aggressive):
+        yield candidate
 
-    shortened = None
-    length = None
-    for length in range(max_line_length, actual_length, delta):
-        shortened = _shorten_line(
-            tokens=tokens,
-            source=source,
-            target=target,
-            indentation=indentation,
-            indent_word=indent_word,
-            newline=newline,
-            max_line_length=length,
-            reverse=reverse,
-            aggressive=aggressive)
+    if aggressive:
+        for key_token_strings in SHORTEN_OPERATOR_GROUPS:
+            shortened = _shorten_line_at_tokens(
+                tokens=tokens,
+                source=source,
+                indentation=indentation,
+                indent_word=indent_word,
+                newline=newline,
+                key_token_strings=key_token_strings,
+                aggressive=aggressive)
 
-        if shortened is not None:
-            break
-
-    if aggressive and (length is None or length > max_line_length):
-        commas_shortened = _shorten_line_at_commas(
-            tokens=tokens,
-            source=source,
-            indentation=indentation,
-            indent_word=indent_word,
-            newline=newline)
-
-        if commas_shortened is not None and commas_shortened != source:
-            shortened = commas_shortened
-
-    return shortened
+            if shortened is not None and shortened != source:
+                yield shortened
 
 
-def _shorten_line(tokens, source, target, indentation, indent_word, newline,
-                  max_line_length, reverse=False, aggressive=False):
-    """Separate line at OPERATOR."""
-    max_line_length_minus_indentation = max_line_length - len(indentation)
-    if reverse:
-        tokens = reversed(tokens)
+def _shorten_line(tokens, source, indentation, indent_word, newline,
+                  aggressive=False):
+    """Separate line at OPERATOR.
+
+    Multiple candidates will be yielded.
+
+    """
     for tkn in tokens:
         # Don't break on '=' after keyword as this violates PEP 8.
         if token.OP == tkn[0] and tkn[1] != '=':
+            assert tkn[0] != token.INDENT
+
             offset = tkn[2][1] + 1
-            if reverse:
-                if offset > (max_line_length_minus_indentation -
-                             len(indent_word)):
-                    continue
-            else:
-                if (len(target.rstrip()) - offset >
-                        (max_line_length_minus_indentation -
-                         len(indent_word))):
-                    continue
-            first = source[:offset - len(indentation)]
+            first = source[:offset]
 
             second_indent = indentation
             if first.rstrip().endswith('('):
@@ -1055,16 +1006,10 @@ def _shorten_line(tokens, source, target, indentation, indent_word, newline,
             else:
                 second_indent += indent_word
 
-            second = (second_indent +
-                      source[offset - len(indentation):].lstrip())
+            second = (second_indent + source[offset:].lstrip())
             if not second.strip():
                 continue
 
-            # Don't modify if lines are not short enough
-            if len(first) > max_line_length_minus_indentation:
-                continue
-            if len(second) > max_line_length:  # Already includes indentation
-                continue
             # Do not begin a line with a comma
             if second.lstrip().startswith(','):
                 continue
@@ -1079,32 +1024,53 @@ def _shorten_line(tokens, source, target, indentation, indent_word, newline,
             # Only fix if syntax is okay.
             if check_syntax(normalize_multiline(fixed)
                             if aggressive else fixed):
-                return indentation + fixed
-    return None
+                yield indentation + fixed
 
 
-def _shorten_line_at_commas(tokens, source, indentation, indent_word, newline):
-    """Separate line by breaking at commas."""
-    if ',' not in source:
-        return None
+def _shorten_line_at_tokens(tokens, source, indentation, indent_word, newline,
+                            key_token_strings, aggressive):
+    """Separate line by breaking at tokens in key_token_strings.
 
-    fixed = ''
+    This will always break the line at the first parenthesis.
+
+    """
+    offsets = []
+    first_paren = True
     for tkn in tokens:
         token_type = tkn[0]
         token_string = tkn[1]
+        next_offset = tkn[2][1] + 1
 
-        if token_string == '.':
-            fixed = fixed.rstrip()
+        assert token_type != token.INDENT
 
-        fixed += token_string
+        if token_string in key_token_strings or (first_paren and
+                                                 token_string == '('):
+            # Don't split right before newline.
+            if next_offset < len(source) - 1:
+                offsets.append(next_offset)
 
-        if token_type == token.OP and token_string == ',':
-            fixed += newline + indent_word
-        elif token_type not in (token.NEWLINE, token.ENDMARKER):
-            if token_string != '.':
-                fixed += ' '
+            if token_string == '(':
+                first_paren = False
 
-    if check_syntax(fixed):
+    current_indent = None
+    fixed = None
+    for line in split_at_offsets(source, offsets):
+        if fixed:
+            fixed += newline + current_indent + line
+
+            for symbol in '([{':
+                if line.endswith(symbol):
+                    current_indent += indent_word
+        else:
+            # First line.
+            fixed = line
+            assert not current_indent
+            current_indent = indent_word
+
+    assert fixed is not None
+
+    if check_syntax(normalize_multiline(fixed)
+                    if aggressive > 1 else fixed):
         return indentation + fixed
     else:
         return None
@@ -1123,6 +1089,9 @@ def normalize_multiline(line):
                 line += '}'
             return '{' + line
 
+    if line.startswith('def ') and line.rstrip().endswith(':'):
+        return line[len('def'):].strip().rstrip(':')
+
     return line
 
 
@@ -1137,13 +1106,6 @@ def fix_whitespace(line, offset, replacement):
         return left + replacement + right
 
 
-def _spawn_pep8(pep8_options):
-    """Execute pep8 via subprocess.Popen."""
-    p = Popen([PEP8_BIN] + pep8_options, stdout=PIPE)
-    output = p.communicate()[0].decode('utf-8')
-    return [_analyze_pep8result(l) for l in output.splitlines()]
-
-
 def _execute_pep8(pep8_options, source):
     """Execute pep8 via python method calls."""
     class QuietReport(pep8.BaseReport):
@@ -1608,42 +1570,53 @@ def refactor_with_2to3(source_text, fixer_name):
     return unicode(tool.refactor_string(source_text, name=''))
 
 
-def break_multi_line(source_text, newline, indent_word, max_line_length):
+def break_multi_line(source_text, newline, indent_word):
     """Break first line of multi-line code.
 
     Return None if a break is not possible.
 
     """
+    indentation = _get_indentation(source_text)
+
     # Handle special case only.
     for symbol in '([{':
         # Only valid if symbol is not on a line by itself.
         if (symbol in source_text
                 and source_text.rstrip().endswith(',')
                 and not source_text.lstrip().startswith(symbol)):
+
             index = 1 + source_text.find(symbol)
-            if index >= max_line_length:
-                return None
 
-            # Make sure we are not in a string.
-            for quote in ['"', "'"]:
-                if quote in source_text:
-                    if source_text.find(quote) < index:
-                        return None
+            if index <= len(indent_word) + len(indentation):
+                continue
 
-            # Make sure we are not in a comment.
-            if '#' in source_text:
-                if source_text.find('#') < index:
-                    return None
+            if is_probably_inside_string_or_comment(source_text, index - 1):
+                continue
 
-            assert index < len(source_text)
             return (
                 source_text[:index].rstrip() + newline +
-                _get_indentation(source_text) + indent_word +
+                indentation + indent_word +
                 source_text[index:].lstrip())
 
     return None
 
 
+def is_probably_inside_string_or_comment(line, index):
+    """Return True if index may be inside a string or comment."""
+    # Make sure we are not in a string.
+    for quote in ['"', "'"]:
+        if quote in line:
+            if line.find(quote) <= index:
+                return True
+
+    # Make sure we are not in a comment.
+    if '#' in line:
+        if line.find('#') <= index:
+            return True
+
+    return False
+
+
 def check_syntax(code):
     """Return True if syntax is okay."""
     try:
@@ -1846,12 +1819,7 @@ def fix_file(filename, options=None, output=None):
     if options.in_place:
         encoding = detect_encoding(filename)
 
-    interruption = None
-    try:
-        fixed_source = fix_lines(fixed_source, options, filename=filename)
-    except KeyboardInterrupt as exception:
-        # Allow stopping early.
-        interruption = exception
+    fixed_source = fix_lines(fixed_source, options, filename=filename)
 
     if options.diff:
         new = StringIO(fixed_source)
@@ -1872,9 +1840,6 @@ def fix_file(filename, options=None, output=None):
         else:
             return fixed_source
 
-    if interruption:
-        raise interruption
-
 
 def parse_args(args):
     """Parse command-line options."""
@@ -1895,25 +1860,30 @@ def parse_args(args):
     parser.add_option('-r', '--recursive', action='store_true',
                       help='run recursively; must be used with --in-place or '
                            '--diff')
-    parser.add_option('--exclude', metavar='globs',
-                      help='exclude files/directories that match these '
-                           'comma-separated globs')
+    parser.add_option('-j', '--jobs', type=int, metavar='n', default=1,
+                      help='number of parallel jobs; '
+                           'match CPU count if value is less than 1')
     parser.add_option('-p', '--pep8-passes', metavar='n',
                       default=100, type=int,
                       help='maximum number of additional pep8 passes'
                            ' (default: %default)')
+    parser.add_option('-a', '--aggressive', action='count', default=0,
+                      help='enable possibly unsafe changes (E711, E712); '
+                           'multiple -a result in more aggressive changes')
+    parser.add_option('--exclude', metavar='globs',
+                      help='exclude files/directories that match these '
+                           'comma-separated globs')
     parser.add_option('--list-fixes', action='store_true',
                       help='list codes for fixes; '
                            'used by --ignore and --select')
     parser.add_option('--ignore', metavar='errors', default='',
-                      help='do not fix these errors/warnings (e.g. E4,W)')
+                      help='do not fix these errors/warnings '
+                           '(default {0})'.format(pep8.DEFAULT_IGNORE))
     parser.add_option('--select', metavar='errors', default='',
                       help='fix only these errors/warnings (e.g. E4,W)')
     parser.add_option('--max-line-length', metavar='n', default=79, type=int,
                       help='set maximum allowed line length '
                            '(default: %default)')
-    parser.add_option('--aggressive', action='store_true',
-                      help='enable possibly unsafe changes (E711, E712)')
     options, args = parser.parse_args(args)
 
     if not len(args) and not options.list_fixes:
@@ -1936,24 +1906,35 @@ def parse_args(args):
     if options.in_place and options.diff:
         parser.error('--in-place and --diff are mutually exclusive')
 
-    if options.max_line_length < 8:
-        parser.error('--max-line-length must greater than 8')
+    if options.max_line_length <= 0:
+        parser.error('--max-line-length must be greater than 0')
 
     if args == ['-'] and (options.in_place or options.recursive):
         parser.error('--in-place or --recursive cannot be used with '
                      'standard input')
 
-    if options.ignore:
-        options.ignore = options.ignore.split(',')
-
     if options.select:
         options.select = options.select.split(',')
 
+    if options.ignore:
+        options.ignore = options.ignore.split(',')
+    elif not options.select and pep8.DEFAULT_IGNORE:
+        options.ignore = pep8.DEFAULT_IGNORE.split(',')
+
     if options.exclude:
         options.exclude = options.exclude.split(',')
     else:
         options.exclude = []
 
+    if options.jobs < 1:
+        # Do not import multiprocessing globally in case it is not supported
+        # on the platform.
+        import multiprocessing
+        options.jobs = multiprocessing.cpu_count()
+
+    if options.jobs > 1 and not options.in_place:
+        parser.error('parallel jobs requires --in-place')
+
     return options, args
 
 
@@ -1972,6 +1953,84 @@ def supported_fixes():
                           getattr(instance, attribute).__doc__))
 
 
+def line_shortening_rank(candidate, newline, indent_word):
+    """Return rank of candidate.
+
+    This is for sorting candidates.
+
+    """
+    rank = 0
+    if candidate:
+        lines = candidate.split(newline)
+
+        offset = 0
+        if lines[0].rstrip()[-1] not in '([{':
+            for symbol in '([{':
+                offset = max(offset, 1 + lines[0].find(symbol))
+
+        max_length = max([offset + len(x.strip()) for x in lines])
+        rank += max_length
+        rank += len(lines)
+
+        bad_staring_symbol = {
+            '(': ')',
+            '[': ']',
+            '{': '}'}.get(lines[0][-1], None)
+
+        if len(lines) > 1:
+            if (bad_staring_symbol and
+                    lines[1].lstrip().startswith(bad_staring_symbol)):
+                rank += 20
+            else:
+                rank -= 10
+
+        if lines[0].endswith('(['):
+            rank += 10
+
+        for current_line in lines:
+            for bad_start in ['.', '%', '+', '-', '/']:
+                if current_line.startswith(bad_start):
+                    rank += 100
+
+            for ending in '([{':
+                # Avoid lonely opening. They result in longer lines.
+                if (current_line.endswith(ending) and
+                        len(current_line.strip()) <= len(indent_word)):
+                    rank += 100
+
+            if current_line.endswith('%'):
+                rank -= 20
+    else:
+        rank = 100000
+
+    return max(0, rank)
+
+
+def split_at_offsets(line, offsets):
+    """Split line at offsets.
+
+    Return list of strings.
+
+    """
+    result = []
+
+    previous_offset = 0
+    current_offset = 0
+    for current_offset in sorted(offsets):
+        if current_offset < len(line) and previous_offset != current_offset:
+            result.append(line[previous_offset:current_offset])
+        previous_offset = current_offset
+
+    result.append(line[current_offset:])
+
+    return result
+
+
+def get_longest_length(text, newline):
+    """Return length of longest line."""
+    return max([len(line) for line in text.split(newline)])
+
+
 class LineEndingWrapper(object):
 
     r"""Replace line endings to work with sys.stdout.
@@ -1999,7 +2058,7 @@ def temporary_file():
         return tempfile.NamedTemporaryFile(mode='w')
 
 
-def match_file(filename, options):
+def match_file(filename, exclude):
     """Return True if file is okay for modifying/recursing."""
     if not filename.endswith('.py'):
         return False
@@ -2007,72 +2066,94 @@ def match_file(filename, options):
     if filename.startswith('.'):
         return False
 
-    for pattern in options.exclude:
+    for pattern in exclude:
         if fnmatch.fnmatch(filename, pattern):
             return False
 
     return True
 
 
+def find_files(filenames, recursive, exclude):
+    """Yield filenames."""
+    while filenames:
+        name = filenames.pop(0)
+        if recursive and os.path.isdir(name):
+            for root, directories, children in os.walk(name):
+                filenames += [os.path.join(root, f) for f in children
+                              if match_file(f, exclude)]
+                for d in directories:
+                    if d.startswith('.'):
+                        directories.remove(d)
+        else:
+            yield name
+
+
+def _fix_file(parameters):
+    """Helper function for optionally running fix_file() in parallel."""
+    if parameters[1].verbose:
+        print('[file:{0}]'.format(parameters[0]), file=sys.stderr)
+    try:
+        fix_file(*parameters)
+    except IOError as error:
+        print(str(error), file=sys.stderr)
+
+
 def fix_multiple_files(filenames, options, output=None):
     """Fix list of files.
 
     Optionally fix files recursively.
 
     """
-    while filenames:
-        name = filenames.pop(0)
-        if options.recursive and os.path.isdir(name):
-            for root, directories, children in os.walk(name):
-                filenames += [os.path.join(root, f) for f in children
-                              if match_file(f, options)]
-                for d in directories:
-                    if d.startswith('.'):
-                        directories.remove(d)
-        else:
-            if options.verbose:
-                print('[file:%s]' % name, file=sys.stderr)
-            try:
-                fix_file(name, options, output)
-            except IOError as error:
-                print(str(error), file=sys.stderr)
+    filenames = find_files(filenames, options.recursive, options.exclude)
+    if options.jobs > 1:
+        import multiprocessing
+        pool = multiprocessing.Pool(options.jobs)
+        pool.map(_fix_file,
+                 [(name, options) for name in filenames])
+    else:
+        for name in filenames:
+            _fix_file((name, options, output))
 
 
 def main():
     """Tool main."""
-    options, args = parse_args(sys.argv[1:])
+    if not pep8:
+        print('pep8 >= 1.3.2 required', file=sys.stderr)
+        return 1
 
-    if options.list_fixes:
-        for code, description in supported_fixes():
-            print('{code} - {description}'.format(
-                code=code, description=description))
-        return 0
+    try:
+        options, args = parse_args(sys.argv[1:])
 
-    if options.in_place or options.diff:
-        filenames = list(set(args))
-    else:
-        assert len(args) == 1
-        assert not options.recursive
-        if args == ['-']:
-            assert not options.in_place
-            temp = temporary_file()
-            temp.write(sys.stdin.read())
-            temp.flush()
-            filenames = [temp.name]
+        if options.list_fixes:
+            for code, description in supported_fixes():
+                print('{code} - {description}'.format(
+                    code=code, description=description))
+            return 0
+
+        if options.in_place or options.diff:
+            filenames = list(set(args))
         else:
-            filenames = args[:1]
+            assert len(args) == 1
+            assert not options.recursive
+            if args == ['-']:
+                assert not options.in_place
+                temp = temporary_file()
+                temp.write(sys.stdin.read())
+                temp.flush()
+                filenames = [temp.name]
+            else:
+                filenames = args[:1]
 
-    output = codecs.getwriter('utf-8')(sys.stdout.buffer
-                                       if sys.version_info[0] >= 3
-                                       else sys.stdout)
+        output = codecs.getwriter('utf-8')(sys.stdout.buffer
+                                           if sys.version_info[0] >= 3
+                                           else sys.stdout)
 
-    output = LineEndingWrapper(output)
+        output = LineEndingWrapper(output)
 
-    fix_multiple_files(filenames, options, output)
+        fix_multiple_files(filenames, options, output)
+    except KeyboardInterrupt:
+        return 1
 
 
 if __name__ == '__main__':
-    try:
-        sys.exit(main())
-    except KeyboardInterrupt:
-        sys.exit(1)
+    sys.exit(main())
