@@ -97,32 +97,36 @@ class TestUtils(unittest.TestCase):
             'foo_bar_zap_bing_bang_boom(\n    111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333,\n',
             autopep8.break_multi_line(
                 'foo_bar_zap_bing_bang_boom(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333,\n',
-                newline='\n', indent_word='    ',
-                max_line_length=79))
+                newline='\n', indent_word='    '))
 
-    def test_break_multi_line_should_not_break_too_long_line(self):
+    def test_break_multi_line_with_long_function(self):
+        self.assertEqual(
+            'foo_bar_zap_bing_bang_boom_foo_bar_zap_bing_bang_boom_foo_bar_zap_bing_bang_boom(\n'
+            '    333,\n',
+            autopep8.break_multi_line(
+                'foo_bar_zap_bing_bang_boom_foo_bar_zap_bing_bang_boom_foo_bar_zap_bing_bang_boom(333,\n',
+                newline='\n', indent_word='    '))
+
+    def test_break_multi_line_should_not_break_too_short_line(self):
         self.assertEqual(
             None,
             autopep8.break_multi_line(
-                'foo_bar_zap_bing_bang_boom_foo_bar_zap_bing_bang_boom_foo_bar_zap_bing_bang_boom(333,\n',
-                newline='\n', indent_word='    ',
-                max_line_length=79))
+                'fo(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333, 333,\n',
+                newline='\n', indent_word='    '))
 
     def test_break_multi_line_should_not_modify_comment(self):
         self.assertEqual(
             None,
             autopep8.break_multi_line(
                 '# foo_bar_zap_bing_bang_boom(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333,\n',
-                newline='\n', indent_word='    ',
-                max_line_length=79))
+                newline='\n', indent_word='    '))
 
     def test_break_multi_line_should_not_modify_lonely_brace(self):
         self.assertEqual(
             None,
             autopep8.break_multi_line(
                 '(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333, 222, 222, 222, 222, 333,\n',
-                newline='\n', indent_word='    ',
-                max_line_length=79))
+                newline='\n', indent_word='    '))
 
     def test_multiline_string_lines(self):
         self.assertEqual(
@@ -288,6 +292,37 @@ def foo():
         self.assertFalse(autopep8.code_match('E26', select='W', ignore=''))
         self.assertFalse(autopep8.code_match('E26', select='W', ignore='E1'))
 
+    def test_split_at_offsets(self):
+        self.assertEqual([''], autopep8.split_at_offsets('', [0]))
+        self.assertEqual(['1234'], autopep8.split_at_offsets('1234', [0]))
+        self.assertEqual(['1', '234'], autopep8.split_at_offsets('1234', [1]))
+        self.assertEqual(['12', '34'], autopep8.split_at_offsets('1234', [2]))
+        self.assertEqual(['12', '3', '4'],
+                         autopep8.split_at_offsets('1234', [2, 3]))
+
+    def test_split_at_offsets_with_out_of_order(self):
+        self.assertEqual(['12', '3', '4'],
+                         autopep8.split_at_offsets('1234', [3, 2]))
+
+    def test_is_probably_inside_string_or_comment(self):
+        self.assertTrue(autopep8.is_probably_inside_string_or_comment(
+            '# abc', 1))
+
+        self.assertFalse(autopep8.is_probably_inside_string_or_comment(
+            'hello # abc', 1))
+
+        self.assertTrue(autopep8.is_probably_inside_string_or_comment(
+            '"abc"', 1))
+
+        self.assertFalse(autopep8.is_probably_inside_string_or_comment(
+            'hello "abc"', 1))
+
+        self.assertTrue(autopep8.is_probably_inside_string_or_comment(
+            '"abc"', 0))
+
+        self.assertFalse(autopep8.is_probably_inside_string_or_comment(
+            ' "abc"', 0))
+
 
 class TestFixPEP8Error(unittest.TestCase):
 
@@ -623,8 +658,8 @@ sql = 'update %s set %s %s' % (from_table,
         fixed = """
 
 sql = 'update %s set %s %s' % (from_table,
-                               ','.join(
-                                   ['%s=%s' % (col, col) for col in cols]),
+                               ','.join(['%s=%s' % (
+                                   col, col) for col in cols]),
                                where_clause)
 """
         with autopep8_context(line) as result:
@@ -1057,7 +1092,7 @@ class Foo():
     def test_e225(self):
         line = "1+1\n2 +2\n3+ 3\n"
         fixed = "1 + 1\n2 + 2\n3 + 3\n"
-        with autopep8_context(line) as result:
+        with autopep8_context(line, options=['--select=E,W']) as result:
             self.assertEqual(fixed, result)
 
     def test_e225_with_indentation_fix(self):
@@ -1080,6 +1115,18 @@ class Foo(object):
         with autopep8_context(line, options=['--select=E22']) as result:
             self.assertEqual(fixed, result)
 
+    def test_e227(self):
+        line = "1&1\n2&2\n3&3\n"
+        fixed = "1 & 1\n2 & 2\n3 & 3\n"
+        with autopep8_context(line, options=['--select=E22']) as result:
+            self.assertEqual(fixed, result)
+
+    def test_e228(self):
+        line = "1%1\n2%2\n3%3\n"
+        fixed = "1 % 1\n2 % 2\n3 % 3\n"
+        with autopep8_context(line, options=['--select=E22']) as result:
+            self.assertEqual(fixed, result)
+
     def test_e231(self):
         line = "[1,2,3]\n"
         fixed = "[1, 2, 3]\n"
@@ -1278,8 +1325,8 @@ print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333, 333,
 """
         fixed = """
 
-print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333,
-      333, 333, 333)
+print(111, 111, 111, 111, 222, 222, 222, 222,
+      222, 222, 222, 222, 222, 333, 333, 333, 333)
 """
         with autopep8_context(line) as result:
             self.assertEqual(fixed, result)
@@ -1289,21 +1336,22 @@ print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222, 222, 333,
 x = [3244234243234, 234234234324, 234234324, 23424234, 234234234, 234234, 234243, 234243, 234234234324, 234234324, 23424234, 234234234, 234234, 234243, 234243]
 """
         fixed = """\
-x = [3244234243234,
-     234234234324,
-     234234324,
-     23424234,
-     234234234,
-     234234,
-     234243,
-     234243,
-     234234234324,
-     234234324,
-     23424234,
-     234234234,
-     234234,
-     234243,
-     234243]
+x = [
+    3244234243234,
+    234234234324,
+    234234324,
+    23424234,
+    234234234,
+    234234,
+    234243,
+    234243,
+    234234234324,
+    234234324,
+    23424234,
+    234234234,
+    234234,
+    234243,
+    234243]
 """
         with autopep8_context(line, options=['--aggressive']) as result:
             self.assertEqual(fixed, result)
@@ -1314,10 +1362,12 @@ parser.add_argument('source_corpus', help='corpus name/path relative to an nltk_
 parser.add_argument('target_corpus', help='corpus name/path relative to an nltk_data directory')
 """
         fixed = """\
-parser.add_argument('source_corpus',
-                    help='corpus name/path relative to an nltk_data directory')
-parser.add_argument('target_corpus',
-                    help='corpus name/path relative to an nltk_data directory')
+parser.add_argument(
+    'source_corpus',
+    help='corpus name/path relative to an nltk_data directory')
+parser.add_argument(
+    'target_corpus',
+    help='corpus name/path relative to an nltk_data directory')
 """
         with autopep8_context(line, options=['--aggressive']) as result:
             self.assertEqual(fixed, result)
@@ -1337,8 +1387,8 @@ def d():
         fixed = """
 
 def d():
-    print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222,
-          222, 333, 333, 333, 333)
+    print(111, 111, 111, 111, 222, 222, 222, 222,
+          222, 222, 222, 222, 222, 333, 333, 333, 333)
 """
         with autopep8_context(line) as result:
             self.assertEqual(fixed, result)
@@ -1352,8 +1402,8 @@ if True:
         fixed = """
 
 if True:
-    print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222,
-          222, 333, 333, 333, 333)
+    print(111, 111, 111, 111, 222, 222, 222, 222,
+          222, 222, 222, 222, 222, 333, 333, 333, 333)
 """
         with autopep8_context(line, options=['--select=E501']) as result:
             self.assertEqual(fixed, result)
@@ -1381,11 +1431,11 @@ fooooooooooooooooooooooooooooooo000000000000000000000000 = [1,
 def d():
     111 + 111 + 111 + 111 + 111 + 222 + 222 + 222 + 222 + 222 + 222 + 222 + 222 + 222 + 333 + 333 + 333 + 333
 """
-        fixed = """
+        fixed = r"""
 
 def d():
-    111 + 111 + 111 + 111 + 111 + 222 + 222 + 222 + 222 + 222 + 222 + \\
-        222 + 222 + 222 + 333 + 333 + 333 + 333
+    111 + 111 + 111 + 111 + 111 + 222 + 222 + 222 + 222 + \
+        222 + 222 + 222 + 222 + 222 + 333 + 333 + 333 + 333
 """
         with autopep8_context(line) as result:
             self.assertEqual(fixed, result)
@@ -1430,6 +1480,17 @@ looooooooooooooong = foo(
         with autopep8_context(line) as result:
             self.assertEqual(fixed, result)
 
+    def test_e501_avoid_breaking_at_empty_parentheses_if_possible(self):
+        line = """\
+someverylongindenttionwhatnot().foo().bar().baz("and here is a long string 123456789012345678901234567890")
+"""
+        fixed = """\
+someverylongindenttionwhatnot().foo().bar().baz(
+    "and here is a long string 123456789012345678901234567890")
+"""
+        with autopep8_context(line) as result:
+            self.assertEqual(fixed, result)
+
     def test_e501_with_multiple_lines(self):
         line = """
 
@@ -1484,6 +1545,7 @@ if True:
             self.assertEqual(fixed, result)
 
     def test_e501_do_not_begin_line_with_comma(self):
+        self.maxDiff = None
         # This fix is incomplete. (The line is still too long.) But it is here
         # just to confirm that we do not put a comma at the beginning of a
         # line.
@@ -1502,8 +1564,7 @@ def dummy():
         if True:
             if True:
                 object = ModifyAction(
-                    [MODIFY70.text, OBJECTBINDING71.text, COLON72.text],
-                    MODIFY70.getLine(), MODIFY70.getCharPositionInLine())
+                    [MODIFY70.text, OBJECTBINDING71.text, COLON72.text], MODIFY70.getLine(), MODIFY70.getCharPositionInLine())
 """
         with autopep8_context(line) as result:
             self.assertEqual(fixed, result)
@@ -1623,22 +1684,22 @@ models = {
 models = {
     'auth.group': {
         'Meta': {'object_name': 'Group'},
-        'id': (
-            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
+        'id': ('django.db.models.fields.AutoField', [], {
+               'primary_key': 'True'}),
         'name': ('django.db.models.fields.CharField', [], {
                  'unique': 'True', 'max_length': '80'}),
-        'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to':
-                        "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
+        'permissions': ('django.db.models.fields.related.ManyToManyField', [], {
+                        'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
     },
     'auth.permission': {
-        'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together':
-                 "(('content_type', 'codename'),)", 'object_name': 'Permission'},
+        'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')",
+                 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
         'codename': ('django.db.models.fields.CharField', [], {
                      'max_length': '100'}),
         'content_type': ('django.db.models.fields.related.ForeignKey', [], {
                          'to': "orm['contenttypes.ContentType']"}),
-        'id': (
-            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
+        'id': ('django.db.models.fields.AutoField', [], {
+               'primary_key': 'True'}),
         'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
     },
 }
@@ -1667,6 +1728,63 @@ import logging, os, bleach, commonware, urllib2, json, time, requests, urlparse,
         with autopep8_context(line, options=['--select=E501']) as result:
             self.assertEqual(line, result)
 
+    def test_e501_should_not_do_useless_things(self):
+        line = """\
+foo('                                                                            ')
+"""
+        with autopep8_context(line) as result:
+            self.assertEqual(line, result)
+
+    def test_e501_aggressive_with_percent(self):
+        line = """\
+raise MultiProjectException("Ambiguous workspace: %s=%s, %s" % ( varname, varname_path, os.path.abspath(config_filename)))
+"""
+        fixed = """\
+raise MultiProjectException(
+    "Ambiguous workspace: %s=%s, %s" %
+    (varname, varname_path, os.path.abspath(config_filename)))
+"""
+        with autopep8_context(line, options=['--aggressive']) as result:
+            self.assertEqual(fixed, result)
+
+    def test_e501_aggressive_with_def(self):
+        line = """\
+def foo(sldfkjlsdfsdf, kksdfsdfsf,sdfsdfsdf, sdfsdfkdk, szdfsdfsdf, sdfsdfsdfsdlkfjsdlf, sdfsdfddf,sdfsdfsfd, sdfsdfdsf):
+    pass
+"""
+        fixed = """\
+
+
+def foo(sldfkjlsdfsdf, kksdfsdfsf, sdfsdfsdf, sdfsdfkdk,
+        szdfsdfsdf, sdfsdfsdfsdlkfjsdlf, sdfsdfddf, sdfsdfsfd, sdfsdfdsf):
+    pass
+"""
+        with autopep8_context(line, options=['--aggressive']) as result:
+            self.assertEqual(fixed, result)
+
+    def test_e501_more_aggressive_with_def(self):
+        line = """\
+def foo(sldfkjlsdfsdf, kksdfsdfsf,sdfsdfsdf, sdfsdfkdk, szdfsdfsdf, sdfsdfsdfsdlkfjsdlf, sdfsdfddf,sdfsdfsfd, sdfsdfdsf):
+    pass
+"""
+        fixed = """\
+
+
+def foo(
+    sldfkjlsdfsdf,
+    kksdfsdfsf,
+    sdfsdfsdf,
+    sdfsdfkdk,
+    szdfsdfsdf,
+    sdfsdfsdfsdlkfjsdlf,
+    sdfsdfddf,
+    sdfsdfsfd,
+        sdfsdfdsf):
+    pass
+"""
+        with autopep8_context(line, options=['--aggressive', '--aggressive']) as result:
+            self.assertEqual(fixed, result)
+
     def test_e502(self):
         line = "print('abc'\\\n      'def')\n"
         fixed = "print('abc'\n      'def')\n"
@@ -1769,7 +1887,7 @@ def foo():
         bar = bar * bar
         return bar
 """
-        with autopep8_context(line) as result:
+        with autopep8_context(line, options=['--select=E,W']) as result:
             self.assertEqual(fixed, result)
 
     def test_e702_with_semicolon_in_string(self):
@@ -2320,6 +2438,42 @@ class TestOptions(unittest.TestCase):
             with open(filename) as f:
                 self.assertEqual(fixed, f.read())
 
+    @unittest.skipIf(os.getenv('TRAVIS') == 'true',
+                     'Travis currently does not like multiprocessing')
+    def test_parallel_jobs(self):
+        line = "'abc'  \n"
+        fixed = "'abc'\n"
+
+        with temporary_file_context(line) as filename_a:
+            with temporary_file_context(line) as filename_b:
+                p = Popen(list(AUTOPEP8_CMD_TUPLE) +
+                          [filename_a, filename_b, '--jobs=3', '--in-place'])
+                p.wait()
+
+                with open(filename_a) as f:
+                    self.assertEqual(fixed, f.read())
+
+                with open(filename_b) as f:
+                    self.assertEqual(fixed, f.read())
+
+    @unittest.skipIf(os.getenv('TRAVIS') == 'true',
+                     'Travis currently does not like multiprocessing')
+    def test_parallel_jobs_with_automatic_cpu_count(self):
+        line = "'abc'  \n"
+        fixed = "'abc'\n"
+
+        with temporary_file_context(line) as filename_a:
+            with temporary_file_context(line) as filename_b:
+                p = Popen(list(AUTOPEP8_CMD_TUPLE) +
+                          [filename_a, filename_b, '--jobs=0', '--in-place'])
+                p.wait()
+
+                with open(filename_a) as f:
+                    self.assertEqual(fixed, f.read())
+
+                with open(filename_b) as f:
+                    self.assertEqual(fixed, f.read())
+
     def test_in_place_with_empty_file(self):
         line = ''
 
@@ -2406,66 +2560,6 @@ class TestOptions(unittest.TestCase):
             self.assertIn('E101', result)
 
 
-class TestSpawnPEP8Process(unittest.TestCase):
-
-    def test_basic(self):
-        line = "print('abc' )    \n1 * 1\n"
-        fixed = "print('abc')\n1 * 1\n"
-        with autopep8_with_spawned_pep8(line) as result:
-            self.assertEqual(fixed, result)
-
-    def test_verbose(self):
-        line = "print('abc' )    \n1 * 1\n"
-        fixed = "print('abc')\n1 * 1\n"
-        sio = StringIO()
-        with capture_stderr(sio):
-            with autopep8_with_spawned_pep8(
-                    line, options=['--verbose']) as result:
-                self.assertEqual(fixed, result)
-        self.assertIn('compatibility mode', sio.getvalue())
-
-    def test_max_line_length(self):
-        line = "foooooooooooooooooo('abcdefghijklmnopqrstuvwxyz')\n"
-        fixed = "foooooooooooooooooo(\n    'abcdefghijklmnopqrstuvwxyz')\n"
-        with autopep8_with_spawned_pep8(
-                line, options=['--max-line-length=40']) as result:
-            self.assertEqual(fixed, result)
-
-    def test_format_block_comments(self):
-        line = """
-foo(  )
-# abc
-bar()#bizz
-  #blah blah
-    #four space indentation
-if True:
-    1
-""".lstrip()
-        fixed = """
-foo()
-# abc
-bar()  # bizz
-  # blah blah
-    # four space indentation
-if True:
-    1
-""".lstrip()
-        with autopep8_with_spawned_pep8(line) as result:
-            self.assertEqual(fixed, result)
-
-    def test_pep8_ignore(self):
-        line = "'abc'  \n"
-        with autopep8_with_spawned_pep8(
-                line, options=['--ignore=E,W']) as result:
-            self.assertEqual(line, result)
-
-    def test_pep8_select(self):
-        line = "'abc'  \n"
-        with autopep8_with_spawned_pep8(
-                line, options=['--select=E101']) as result:
-            self.assertEqual(line, result)
-
-
 class TestCoverage(unittest.TestCase):
 
     def test_fixpep8_class_constructor(self):
@@ -2543,23 +2637,6 @@ def autopep8_subprocess(line, options=None):
         yield p.communicate()[0].decode('utf-8')
 
 
-@contextlib.contextmanager
-def autopep8_with_spawned_pep8(line, options=None):
-    if not options:
-        options = []
-
-    with temporary_file_context(line) as filename:
-        options, _ = autopep8.parse_args(list(options) + [filename])
-
-        # Monkey patch pep8 to trigger spawning
-        original_pep8 = autopep8.pep8
-        try:
-            autopep8.pep8 = None
-            yield autopep8.fix_file(filename=filename, options=options)
-        finally:
-            autopep8.pep8 = original_pep8
-
-
 @contextlib.contextmanager
 def temporary_file_context(text):
     tempfile = mkstemp()
